import grails.util.GrailsUtil
import org.codehaus.groovy.grails.commons.GrailsApplication
import org.springframework.web.util.UrlPathHelper
import javax.servlet.http.HttpServletRequest
import org.codehaus.groovy.grails.commons.ConfigurationHolder

class DebugFilters {
    def filters =  {
        debugGlobalFilter(controller:'*', action:'*') {
                before = {
                    if (getWeAreDebugging()) {
                        request[DebugService.ATTRIB_REQUEST_START] = System.currentTimeMillis()
                        
                        def debugService = applicationContext.debugService
                        debugService.traceVar(request, 'stats', 'requestStart', new Date(request[DebugService.ATTRIB_REQUEST_START]))

                        if (isDebugEnabled()) {
                            log.info("Request received from ${request.remoteAddr}")
                        }
                    }
                }
                
                after = { 
                    if (getWeAreDebugging()) {
                        
                        def debugService = applicationContext.debugService
                        def now = System.currentTimeMillis()
                        debugService.traceVar(request, 'stats', 'requestHandled', new Date(now))
                    }
                }
                
                afterView = {
                    if (getWeAreDebugging()) {
                        
                        def debugService = applicationContext.debugService
                        def now = System.currentTimeMillis()
                        debugService.traceVar(request, 'stats', 'requestRendered', new Date(now))

                        if (isDebugEnabled()) {
                            debugService.collect(request)
                            log.info("Total request time: ${now - request[DebugService.ATTRIB_REQUEST_START]}ms")
                        }
                    }
                }
        }
    }
    
    private isDebugEnabled() {
        // This logic is hassle - to handle defaulting to on if no value specified, or value is blank,
        // but only turn OFF if it is set to false
        def logging = 'false' != (ConfigurationHolder.config.grails.debug.enabled ?: '')
    }
    
    // Work out if we should even attempt to log/debug anything at all
    private getWeAreDebugging() {
        return (GrailsUtil.environment != GrailsApplication.ENV_PRODUCTION) || 
            ConfigurationHolder.config.grails.debug.productionOverride
    }
}